#include "dr26.hpp"

void ForteDR26 ::setup()
{
	Wire.begin(6, 7);
  //  ads.setGain(GAIN_ONE);
  //  ads.begin() ?  Serial.println("ADS initialized") : Serial.println("failed to initialize ADS");


    ads1.setGain(GAIN_ONE);
    ads1.begin() ?  Serial.println("ADS initialized") : Serial.println("failed to initialize ADS");
    delay(100);
    channel=0;
}

float ForteDR26 ::readData()
{
	float volts=0;
    for(int i=0; i<10; i++){
        int16_t adc = 0;
        float volt = 0;
        try
        {
            adc = ads1.readADC_SingleEnded(channel);
            volt = ads1.computeVolts(adc);
        }
        catch(NoDataAvailableException& e)
        {
	        throw NoDataAvailableException();  //propagate exception
        }
        volts += volt;
    }

    volts /= 10;
    return volts;
}

// The following functions change the ADC input range: be careful
// to never to exceed VDD +0.3V max, or to exceed the upper and
// lower limits if you adjust the input range.
// SETTING THE GAIN VALUE INCORRECTLY MAY DESTROY THE ADC!
// The maximum output of the dendrometer is 2.5V, and so a gain of
// one (max 4.096V) or two (max 2.048V) is optimal. Changing the gain
// changes the accuracy of the sensor: higher gain gives a higher
// precision but a smaller range.
//
// GAIN_TWOTHIRDS  // 2/3x gain +/- 6.144V  1 bit = 0.1875mV (default)
// GAIN_ONE        // 1x gain   +/- 4.096V  1 bit = 0.125mV
// GAIN_TWO        // 2x gain   +/- 2.048V  1 bit = 0.0625mV
// GAIN_FOUR       // 4x gain   +/- 1.024V  1 bit = 0.03125mV
// GAIN_EIGHT      // 8x gain   +/- 0.512V  1 bit = 0.015625mV
// GAIN_SIXTEEN    // 16x gain  +/- 0.256V  1 bit = 0.0078125mV
void ForteDR26 ::changeGain(adsGain_t gain)
{
	ads.setGain(gain);
}

void ForteDR26 ::setChannel(int c)
{
	channel=c;
}

std::list<Message> ForteDR26::buildMessages()
{
	std::list<Message> messages;
	float data = readData();
	MeasurementData IncrementData{data, 0, {}, "CIRCUMFERENCE_INCREMENT"};
	messages.emplace_back(IncrementData, sensorInformation, Time::getInstance().getEpochSeconds());
	return messages;
}
SensorInformation ForteDR26::getSensorInformation() const
{
	return sensorInformation;
}


Adafruit_ADS1115 ForteDR26::ads1=ads1;