#pragma once

#include <ArduinoJson.h>
#include "MeasurementData.hpp"
#include "Protocol.hpp"
#include "SensorInformation.hpp"
#include <list>
#include <optional>
#include <string>
#include <utility>

// having the data be a struct of basic types makes sending easier,
// otherwise we would have to serialize the data before sending
class ClientDataPackage {
  private:
	MeasurementData measurementData;
	SensorInformation sensorInformation;
	unsigned long timestamp; // maybe make this array

  public:
	ClientDataPackage(MeasurementData value, SensorInformation sensorInformation, unsigned long timestamp)
	    : measurementData(std::move(value)), sensorInformation(std::move(sensorInformation)), timestamp(timestamp)
	{
	}

	[[nodiscard]] const MeasurementData &getMeasurementData() const { return measurementData; }
	[[nodiscard]] const SensorInformation &getSensorInformation() const { return sensorInformation; }
	[[nodiscard]] unsigned long getTimestamp() const { return timestamp; }

	[[nodiscard]] std::string getDataPackageAsMinifiedJsonString() const
	{
		StaticJsonDocument<250> document; // 250 byte is the max send size of espnow

		document["sensorName"] = sensorInformation.getSensorName();
		document["timestamp"] = timestamp;
		document["protocol"] = protocolToString.at(sensorInformation.getProtocol());
		document["value"] = measurementData.getValue();

		if (measurementData.getChannel().has_value()) {
			document["channel"] = measurementData.getChannel().value();
		}

		if (measurementData.getI2CAddress().has_value()) {
			document["i2cAddress"] = measurementData.getI2CAddress().value();
		}

		document["measurementType"] = measurementData.getMeasurementType();

		std::string jsonString;
		serializeJson(document, jsonString);
		return jsonString;
	}

};
