#include "Message.hpp"

#include <utility>

static const char *TAG = "MESSAGE";

esp_err_t Message::send() const
{
	ESP_LOGI(TAG, "Sending message");
	esp_err_t success;
	auto messageData = getMessageAsMinifiedJsonString();

	// conversion from std::string to c_str adds null terminator, which is why we add 1 to message length
	success = esp_now_send(recipient, (uint8_t *)messageData.c_str(), (messageData.length() + 1) * sizeof(char));
	if (success != ESP_OK) {
		ESP_LOGE(TAG, "Error sending the data");
		Serial.println(success, HEX);
		// Removed caching from here, better do this in main
	}
	ESP_LOGE(TAG, "Sent data: %s", messageData.c_str());

	ESP_LOGD(TAG, "time sent: %l", clientDataPackage.getTimestamp());
	ESP_LOGD(TAG, "send status: %d", success);

	return success;
}

std::string Message::getMessageAsMinifiedJsonString() const
{
	return clientDataPackage.getDataPackageAsMinifiedJsonString();
}

Message::Message(ClientDataPackage data) : clientDataPackage(std::move(data))
{
	// check for existing host mac address, use broadcast otherwise
	get_host_mac(recipient);
}
Message::Message(MeasurementData const &data, const SensorInformation &information, unsigned long timestamp)
    : clientDataPackage(data, information, timestamp)
{
	// check for existing host mac address, use broadcast otherwise
	get_host_mac(recipient);
}
