#include "scd30.hpp"

void ForteSCD30 ::setup()
{
	Wire.begin(I2C_SDA, I2C_SCL);
	if (!airSensor.begin()) {
		// Sensor init went wrong
		ESP_LOGW(sensorInformation.getSensorName().c_str(), "Initialization failed.");
		return;
	}
}

out_data_scd30 ForteSCD30 ::readData()
{
	if (airSensor.dataAvailable()) {
		data.C02 = airSensor.getCO2();
		data.Temperature = airSensor.getTemperature();
		data.Humidity = airSensor.getHumidity();

		return data;
	}
	throw NoDataAvailableException();
}

std::list<Message> ForteSCD30::buildMessages()
{
	std::list<Message> messages;

	out_data_scd30 data = readData();
	MeasurementData CO2Data{data.C02, 0, {}, "CO2"};
	MeasurementData TempData{data.Temperature, 0, {}, "Temperature"};
	MeasurementData HumidData{data.Humidity, 0, {}, "Humidity"};

	messages.emplace_back(Message(CO2Data, sensorInformation, Time::getInstance().getEpochSeconds()));
	messages.emplace_back(Message(TempData, sensorInformation, Time::getInstance().getEpochSeconds()));
	messages.emplace_back(Message(HumidData, sensorInformation, Time::getInstance().getEpochSeconds()));
	
	return messages;
}

SensorInformation ForteSCD30::getSensorInformation() const
{
	return sensorInformation;
}
