#ifndef SENTECSENSORS_H
#define SENTECSENSORS_H

#include <Arduino.h>
#include <SoftwareSerial.h>

class SentecSensorRS485
{
public:
    byte address;
    uint8_t serialCommunicationControlPin = 6;
    SoftwareSerial *RS485;
    byte answerFrame[10];
    // TODO use valid flag to log None
    bool valid = false;
    SentecSensorRS485(SoftwareSerial *ser, byte add);

    void write(byte queryFrame[], int length);
    String getValueStr(float value);
    String getValueStr(int value);
    void queryAddress();
    void readRegister(int registerStartAddress);
    void readRegister(int registerStartAddress, int registerLength);
    void writeRegister(int registerAddress, int value);
    void setAddress(byte add);
    void resetAnswerFrame();
    bool getResponse();
    unsigned int calculateCRC(byte query[], int length);
    void printBytes(byte *data, int length);
    void printBytes(word data);
};

class SolarRadiationSensor : public SentecSensorRS485
{
public:
    using SentecSensorRS485::SentecSensorRS485;
    // global radiation [W/m^2]
    word glob = 0;

    word getSolarRadiation();
    String getSolarRadiationStr();
};

class RainGaugeSensor : public SentecSensorRS485
{
public:
    using SentecSensorRS485::SentecSensorRS485;
    // precipitation values [mm]
    word precipitation = 0;     // prcp since reset? (I THINK!!!)
    word prcpToday = 0;         // prcp since 00:00
    word prcpInstantaneous = 0; // rainfall since last query - THIS IS OFF
    word prcpYesterday = 0;     // prcp sum yesterday 00:00-24:00
    word prcpTotal = 0;         // prcp since sensor power-up
    word prcpHourly = 0;        // prcp this hour
    word prcpLastHourly = 0;    // prcp last hour
    word prcp24max = 0;         // max mm in the last 24h
    word prcp24maxPeriod = 0;   // time when max prcp was measured
    word prcp24min = 0;         // min mm in the last 24h
    word prcp24minPeriod = 0;   // time when min prcp was measured

    void queryTime();
    void setTime();
    void resetPrecipitation();
    void resetSensor();
    void getPrecipitation();
    word getInstantaneousPrecipitation();
    String getPrecipitationStr();
};

class SoilMoistureSensor : public SentecSensorRS485
{
public:
    using SentecSensorRS485::SentecSensorRS485;
    // vwc: volumetric water content [%]
    word moistureRaw = 0;
    int moistureOffset = 0;
    // soil temperature [deg C]
    int temperatureRaw = 0;
    int temperatureOffset = 0;

    word getMoistureTemp();
    float getMoisture();
    String getMoistureStr();
    String getTemperatureStr();
};


#endif