#ifndef ESPTIME
#define ESPTIME

#include <ESP32Time.h>

class Time {
  public:
	static Time &getInstance()
	{
		static Time instance; // Guaranteed to be destroyed.
		                      // Instantiated on first use.
		return instance;
	}

	/*!
	@brief  set the internal RTC time
	@param  epoch
	        epoch time in seconds
	@param  ms
	        microseconds (optional)
	*/
	void setTime(long epoch, int ms = 0);

	/*!
	@brief  get the internal RTC time as a tm struct
	*/
	tm getTimeStruct();

	/*!
	@brief  get the time and date as an Arduino String object
	@param  mode
	        true = Long date format
	        false = Short date format
	*/
	String getDateTime(bool mode);

	/*!
	@brief  get the time as an Arduino String object with the specified format
	@param	format
	        time format
	        http://www.cplusplus.com/reference/ctime/strftime/
	*/
	String getTime(String format = "%H:%M:%S");

	/*!
	@brief  get the current epoch seconds as long
	*/
	long getEpochSeconds();

	/*!
	@brief  get the current milliseconds as long
	*/
	long getMillis();

  private:
	Time() {} // Constructor? (the {} brackets) are needed here.

	ESP32Time rtc = ESP32Time{};

	// C++ 11
	// =======
	// We can use the better technique of deleting the methods
	// we don't want.
  public:
	Time(Time const &) = delete;
	void operator=(Time const &) = delete;

	// Note: Scott Meyers mentions in his Effective Modern
	//       C++ book, that deleted functions should generally
	//       be public as it results in better error messages
	//       due to the compilers behavior to check accessibility
	//       before deleted status
};

#endif