
/*******************************************
   LIBRARIES
   The use of angle brackets (<>) causes the
   compiler to search the default include
   directory. Double quotes ("") causes it
   to search the current working directory
   and, if that search fails, it defaults
   to the default include directory.
*******************************************/

#include <Wire.h>                           // I2C bus
#include "WiFi.h"                           // WiFi connection
#include <SparkFun_SCD30_Arduino_Library.h> // CO2 sensor library

/*******************************************
   VARIABLES & CONSTANTS
*******************************************/

#define I2C_SDA 18 // use GPIO pin for I2C: SDA 
#define I2C_SCL 19 // use GPIO pin for I2C: SCL  
/*******************************************
   INSTANTIATION
*******************************************/

SCD30 airSensor;
WiFiClient client;

void connectToWiFi()
{

    WiFi.mode(WIFI_STA);
    WiFi.begin("micropython", "ThisIsAPassword");
    Serial.print("Connecting to micropython");

    uint8_t i = 0;
    while (WiFi.status() != WL_CONNECTED)
    {
        Serial.print('.');
        delay(500);

        if ((++i % 16) == 0)
        {
            Serial.println(F(" still trying to connect"));
        }
    }

    Serial.print(F("Connected. My IP address is: "));
    Serial.println(WiFi.localIP());
}

void setup()
{
    Serial.begin(115200);
    Serial.println("SCD30 Example");
    Wire.begin(I2C_SDA, I2C_SCL);

    if (airSensor.begin() == false)
    {
        Serial.println("Air sensor not detected. Please check wiring. Freezing...");
        while (true)
        {
        }
    }
    connectToWiFi();
}

void send_data_wifi(String send_string)
{
    // Send data to server
    int conn = client.connect("192.168.4.1", 80);
    if (conn == 1)
    {
        client.println("POST / HTTP/1.1");
        client.println("Host: 192.168.4.1");
        client.println("Content-Type: application/json");
        client.println("Content-Length: " + String(send_string.length()));
        client.println();
        client.println(send_string);
    }
    else
    {
        client.stop();
        Serial.println("Connection failed");
    }
}

void loop()
{
    if (airSensor.dataAvailable())
    {

        auto c02 = airSensor.getCO2();
        Serial.print("co2(ppm):");
        Serial.print(c02);

        auto temp = airSensor.getTemperature();
        Serial.print(" temp(C):");
        Serial.print(temp, 1);

        auto humidity = airSensor.getHumidity();
        Serial.print(" humidity(%):");
        Serial.print(humidity, 1);

        Serial.println();

        String str = "{\"co2\": " + String(c02) + ", \"temp\": " + String(temp) + ", \"humidity\": " + String(humidity) + "}";
        send_data_wifi(str);
        
    }
    else
        Serial.println("Waiting for new data");

    // The SCD30 has data ready every two seconds
    delay(2500);
}