#include <Arduino.h>
#include <NeoPixelBus.h>
#include <esp_now.h>
#include "WiFi.h"

#define PIN 8
#define NUMPIXELS 1
#define LED 0

// uint8_t peerMAC[] = {0x58, 0xCF, 0x79, 0x04, 0x45, 0x9C}; //battery
// uint8_t peerMAC[] = {0x58, 0xCF, 0x79, 0x04, 0x3A, 0xF8}; //wired
uint8_t peerMAC[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

typedef struct espMessage{
  String macAddr;
  int temp;
  int co;
} espMessage;

espMessage lastMessage;

esp_now_peer_info_t peerInfo;

NeoPixelBus<NeoGrbFeature, Neo800KbpsMethod> led(NUMPIXELS, PIN);

void blink()
{
  // RgbColor current = led.GetPixelColor(0);
  // if(current == RgbColor(0,0,0)){
  //   led.SetPixelColor(LED, RgbColor(55, 55, 55));
  //   led.Show();
  //   Serial.println("Led is on at peer" + WiFi.macAddress());
  // }
  // else{
  //   led.SetPixelColor(LED, RgbColor(0,0,0));
  //   led.Show();
  //   Serial.println("led is off");
  // }
}

void flash(RgbColor color){
  led.SetPixelColor(LED, color);
  led.Show();
  delay(500);
  led.SetPixelColor(LED, RgbColor(0,0,0));
  led.Show();
}

void onDataSent(const uint8_t *mac_addr, esp_now_send_status_t status)
{
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Message recieved" : "Message lost");
  flash(RgbColor(255, 255, 255));
}

void onDataRecv(const uint8_t *mac, const uint8_t *incomingData, int len)
{
  Serial.println("message recieved");
  memcpy(&lastMessage, incomingData, sizeof(lastMessage));
  Serial.println(lastMessage.macAddr);
  Serial.println(lastMessage.co);
  Serial.println(lastMessage.temp);
  Serial.println("--------------");
}

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);
  while (!Serial);

  //Led stuff
  led.Begin();
  led.SetPixelColor(LED, RgbColor(0,0,0));
  led.Show();

  //esp now
  WiFi.mode(WIFI_STA);
  if(esp_now_init() != ESP_OK){
    Serial.println("Error initializing espnow");
    flash(RgbColor(0, 0, 55));
    return;
  }

  //comment out whichever you don't need
  esp_now_register_send_cb(onDataSent);
  // esp_now_register_recv_cb(onDataRecv);

  //register peer
  memcpy(peerInfo.peer_addr, peerMAC, sizeof(peerMAC));
  peerInfo.channel = 0;
  peerInfo.encrypt = false;
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    flash(RgbColor(0, 55, 55));
    return;
  }


}

void loop() {
  // put your main code here, to run repeatedly:
  
  //comment this out if you only want to recieve
  espMessage data;
  data.macAddr = WiFi.macAddress();
  data.co = (int) random(5,30);
  data.temp = (int) random(0,100);

  esp_err_t result = esp_now_send(peerMAC, (uint8_t *) &data, sizeof(data));
  if(result == ESP_OK){
    flash(RgbColor(0, 55, 0));
  }
  else{
    flash(RgbColor(55, 0, 0));
    Serial.println(result);
  }

  delay(5000);
}