logfiles_directory = 'logfiles/'

import os
import sys
import time
import datetime
import json
import pandas

# read datalog.txt line by line
with open(logfiles_directory + 'datalog.txt', 'r') as f:
    lines = f.readlines()
    json_list = []
    # parse json for each line
    for line in lines:
        try:
            data = json.loads(line)
            json_list.append(data)
        except:
            print('Error parsing line: ' + line)
# convert json list to pandas dataframe
df = pandas.DataFrame(json_list)
# convert timestamp to datetime
df['timestamp'] = pandas.to_datetime(df['timestamp'], unit='s')
# set timestamp as index
df.set_index('timestamp', inplace=True)
# sort by timestamp
df.sort_index(inplace=True)
# print dataframe
print(df)

# import matplotilb
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages

# plot dataframe where measurementtype temperature. scatter
df['timestamp'] = df.index
plot1 = df[df['measurementType'] == 'TEMPERATURE'][1956:].plot(kind='scatter', x='timestamp',
                                                               y='value',
                                                               title='Temperature (DRS26)',
                                                               figsize=(15, 5),
                                                               grid=True)
plot2 = df[df['measurementType'] == 'TEMPERATURE'][1956:].plot(kind='scatter', x='timestamp', y=0,
                                                               title='Temperature (DRS26) - plot to see missing',
                                                               figsize=(15, 5), grid=True)
# plot dataframe where measurementType CIRCUMFERENCE_INCREMENT and sensorName DRS26.
plot3 = df[(df['measurementType'] == 'CIRCUMFERENCE_INCREMENT') & (df['sensorName'] == 'DRS26')][
        1956:].plot(kind='scatter', x='timestamp', y='value',
                    title='Circumference Increment (DRS26)',
                    figsize=(15, 5), grid=True)
plot4 = df[(df['measurementType'] == 'CIRCUMFERENCE_INCREMENT') & (df['sensorName'] == 'DRS26')][
        1956:].plot(kind='scatter', x='timestamp', y=0,
                    title='Circumference Increment (DRS26) - plot to see missing', figsize=(15, 5),
                    grid=True)
# plot dataframe where measurementType CIRCUMFERENCE_INCREMENT and sensorName DR26.
plot5 = df[(df['measurementType'] == 'CIRCUMFERENCE_INCREMENT') & (df['sensorName'] == 'DR26')][
        1956:].plot(kind='scatter', x='timestamp', y='value',
                    title='Circumference Increment (DR26)',
                    figsize=(15, 5), grid=True)
plot6 = df[(df['measurementType'] == 'CIRCUMFERENCE_INCREMENT') & (df['sensorName'] == 'DR26')][
        1956:].plot(kind='scatter', x='timestamp', y=0,
                    title='Circumference Increment (DR26) - plot to see missing', figsize=(15, 5),
                    grid=True)
# plot dataframe where measurementType PRECIPITATION and sensorName SENTEC_SENSORS.
plot7 = df[(df['measurementType'] == 'PRECIPITATION') & (df['sensorName'] == 'SENTEC_SENSORS')][
        1956:].plot(kind='scatter', x='timestamp', y='value', title='PRECIPITATION (sentec)',
                    figsize=(15, 5), grid=True)
plot8 = df[(df['measurementType'] == 'PRECIPITATION') & (df['sensorName'] == 'SENTEC_SENSORS')][
        1956:].plot(kind='scatter', x='timestamp', y=0,
                    title='PRECIPITATION (sentec) - plot to see missing', figsize=(15, 5),
                    grid=True)
# plot dataframe where measurementType MOISTURE_TEMPERATURE and sensorName SENTEC_SENSORS.
plot9 = df[(df['measurementType'] == 'MOISTURE_TEMPERATURE') & (
            df['sensorName'] == 'SENTEC_SENSORS')][
        1956:].plot(kind='scatter', x='timestamp', y='value', title='MOISTURE_TEMPERATURE (sentec)',
                    figsize=(15, 5), grid=True)
plot10 = df[(df['measurementType'] == 'MOISTURE_TEMPERATURE') & (
            df['sensorName'] == 'SENTEC_SENSORS')][
         1956:].plot(kind='scatter', x='timestamp', y=0,
                     title='MOISTURE_TEMPERATURE (sentec) - plot to see missing', figsize=(15, 5),
                     grid=True)
# plot dataframe where measurementType MOISTURE and sensorName SENTEC_SENSORS.
plot11 = df[(df['measurementType'] == 'MOISTURE') & (df['sensorName'] == 'SENTEC_SENSORS')][
         1956:].plot(kind='scatter', x='timestamp', y='value', title='MOISTURE (sentec)',
                     figsize=(15, 5), grid=True)
plot12 = df[(df['measurementType'] == 'MOISTURE') & (df['sensorName'] == 'SENTEC_SENSORS')][
         1956:].plot(kind='scatter', x='timestamp', y=0,
                     title='MOISTURE (sentec) - plot to see missing', figsize=(15, 5),
                     grid=True)
# plot dataframe where measurementType SOLAR_RADIATION and sensorName SENTEC_SENSORS.
plot13 = df[(df['measurementType'] == 'SOLAR_RADIATION') & (df['sensorName'] == 'SENTEC_SENSORS')][
         1956:].plot(kind='scatter', x='timestamp', y='value', title='SOLAR_RADIATION (sentec)',
                     figsize=(15, 5), grid=True)
plot14 = df[(df['measurementType'] == 'SOLAR_RADIATION') & (df['sensorName'] == 'SENTEC_SENSORS')][
         1956:].plot(kind='scatter', x='timestamp', y=0,
                     title='SOLAR_RADIATION (sentec) - plot to see missing', figsize=(15, 5),
                     grid=True)

# save plots to pdf
with PdfPages('plots.pdf') as pdf:
    pdf.savefig(plot1.get_figure())
    pdf.savefig(plot2.get_figure())
    pdf.savefig(plot3.get_figure())
    pdf.savefig(plot4.get_figure())
    pdf.savefig(plot5.get_figure())
    pdf.savefig(plot6.get_figure())
    pdf.savefig(plot7.get_figure())
    pdf.savefig(plot8.get_figure())
    pdf.savefig(plot9.get_figure())
    pdf.savefig(plot10.get_figure())
    pdf.savefig(plot11.get_figure())
    pdf.savefig(plot12.get_figure())
    pdf.savefig(plot13.get_figure())
    pdf.savefig(plot14.get_figure())

# save df as csv
df.to_csv(logfiles_directory + 'datalog.csv')
